﻿package  {
	
	import com.ad.vpaid.*;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TimerEvent;
	import flash.system.Security;
	import flash.utils.Timer;
	
	public class Creative extends MovieClip implements IVPAID {

		private var _width:Number = 640;
		private var _height:Number = 400;
		
		private var _adVideoPlayer:AdVideoPlayer;
		private var _isUserInteractive:Boolean = false;
		private var _isStartAdCalled:Boolean = false;
		private var _isStopAdCalled:Boolean = false;
		private var _isSkipAdCalled:Boolean = false;
		
		private var _metadataTimeoutTimer:Timer;
		
		public function Creative() {
			// Make this creative accessible for the loader
			Security.allowDomain("*");
			
			_adVideoPlayer = new AdVideoPlayer();
		}

		// Some SDK will call this method first.
		// In our case, just return this.
		public function getVPAID():MovieClip {
			return this;
		}

		private function clean():void {
			cleanPreparation();
			_adVideoPlayer.removeEventListener(VPAIDEvent.AdError, onVideoError);
			_adVideoPlayer.removeEventListener(VPAIDEvent.AdVideoStart, onVideoProgress);
			_adVideoPlayer.removeEventListener(VPAIDEvent.AdVideoFirstQuartile, onVideoProgress);
			_adVideoPlayer.removeEventListener(VPAIDEvent.AdVideoMidpoint, onVideoProgress);
			_adVideoPlayer.removeEventListener(VPAIDEvent.AdVideoThirdQuartile, onVideoProgress);
			_adVideoPlayer.removeEventListener(VPAIDEvent.AdVideoComplete, onVideoProgress);
			
			this.removeChild(_adVideoPlayer.video);
			_adVideoPlayer.clean();
			
			unregisterInteraction();
		}
		
        // ========================
		// Ad Initialization starts
		// ========================
		
		// Load the video.
		// This method will be called in initAd.
		private function prepare():void {
			// If 2 seconds passed after loading the video,
			// and the metadata is yet to be returned,
			// send adLoaded event and ignore metadata.
			_metadataTimeoutTimer = new Timer(2000, 1);
			_metadataTimeoutTimer.addEventListener(TimerEvent.TIMER, onMetadataFailed);
			
			_adVideoPlayer.addEventListener(AdVideoPlayer.METADATA_FETCHED, onMetadataFetched);
			_adVideoPlayer.addEventListener(VPAIDEvent.AdError, onVideoError);
			_adVideoPlayer.addEventListener(VPAIDEvent.AdVideoStart, onVideoProgress);
			_adVideoPlayer.addEventListener(VPAIDEvent.AdVideoFirstQuartile, onVideoProgress);
			_adVideoPlayer.addEventListener(VPAIDEvent.AdVideoMidpoint, onVideoProgress);
			_adVideoPlayer.addEventListener(VPAIDEvent.AdVideoThirdQuartile, onVideoProgress);
			_adVideoPlayer.addEventListener(VPAIDEvent.AdVideoComplete, onVideoProgress);
			_adVideoPlayer.load("http://creativecenter-video.oss-cn-hangzhou.aliyuncs.com/video2039181407750182761.mp4");
			
			_metadataTimeoutTimer.start();
			
			// Init the interactive button
			registerInteraction();
		}
		
		private function cleanPreparation():void {
			if (_metadataTimeoutTimer) {
				_metadataTimeoutTimer.stop();
				_metadataTimeoutTimer.removeEventListener(TimerEvent.TIMER, onMetadataFailed);
				_metadataTimeoutTimer = null;
			}
			_adVideoPlayer.removeEventListener(AdVideoPlayer.METADATA_FETCHED, onMetadataFetched);
		}
		
		private function onVideoError(event:VPAIDEvent):void {
			cleanPreparation();
			_adVideoPlayer.removeEventListener(VPAIDEvent.AdError, onVideoError);
			dispatchEvent(event);
		}
		
		private function onMetadataFailed(event:TimerEvent):void {
			didPrepare();
		}
		
		private function onMetadataFetched(event:Event):void {
			didPrepare();
		}
		
		private function onVideoProgress(event:VPAIDEvent):void {
			dispatchEvent(event);
		}
		
		private function didPrepare():void {
			cleanPreparation();
			this.addChild(_adVideoPlayer.video);
			
			// Usually you scale the video propotionally
			// But I know the file I'm using is 640x480
			// So I just simply set the new size
			_adVideoPlayer.video.width = 533;
			_adVideoPlayer.video.height = 400;
			_adVideoPlayer.video.x = 107;
			_adVideoPlayer.video.y = 0;
			
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdLoaded));
		}
        // ========================
		//  Ad Initialization ends
		// ========================
		
        // ========================
		//    User actions start
		// ========================
		private function registerInteraction():void {
			// sidebar_mc and wrapper_mc are on the stage of the fla file
			sidebar_mc.hover_btn.addEventListener(MouseEvent.ROLL_OVER, startInteraction);
			sidebar_mc.hover_btn.addEventListener(MouseEvent.ROLL_OUT, endInteraction);
			wrapper_mc.addEventListener(MouseEvent.CLICK, onVideoClick);
		}
		
		private function unregisterInteraction():void {
			sidebar_mc.hover_btn.removeEventListener(MouseEvent.ROLL_OVER, startInteraction);
			sidebar_mc.hover_btn.removeEventListener(MouseEvent.ROLL_OUT, endInteraction);
			wrapper_mc.removeEventListener(MouseEvent.CLICK, onVideoClick);
		}
		
		private function startInteraction(event:MouseEvent):void {
			_adVideoPlayer.pause();
			_isUserInteractive = true;
			sidebar_mc.hover_btn.label = "Interacting";
			
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdInteraction, {id: "button_hover"}));
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdDurationChange));
		}
		
		private function endInteraction(event:MouseEvent):void {
			_adVideoPlayer.resume();
			_isUserInteractive = false;
			sidebar_mc.hover_btn.label = "Hover me";
			
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdDurationChange));
		}
		
		private function onVideoClick(event:MouseEvent):void {
			// When you need to open a new page on click,
			// you can ask the player to do so.
			// Or you can do it yourself by setting playerHandles=false
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdClickThru, {
				url: "http://yehao.diandian.com",
				id: "video_click",
				playerHandles: true
			}));
		}
        // ========================
		//     User actions end
		// ========================
		
        // ========================
		// IVPAID Properties start
		// ========================
        public function get adLinear():Boolean {
			// This is a pre-roll linear creative.
			// The adLinear property is not gonna change.
			return true;
		}
		
        public function get adWidth():Number {
			return _width;
		}
		
        public function get adHeight():Number {
			return _height;
		}
		
        public function get adExpanded():Boolean {
			// Cannot expand or collapse.
			// Always return false.
			return false;
		}
		
        public function get adSkippableState():Boolean {
			// Can be skipped all the time.
			// Always return true.
			return true;
		}
		
        public function get adRemainingTime():Number {
			if (_isUserInteractive) {
				return -2;
			}
			
			var remainingTime:Number = _adVideoPlayer.duration - _adVideoPlayer.time;
			if (remainingTime <= 0) {
				remainingTime = 0;
			}
			return remainingTime;
		}
		
        public function get adDuration():Number {
			if (_isUserInteractive) {
				return -2;
			}
			
			return _adVideoPlayer.duration;
		}
		
        public function get adVolume():Number {
			return _adVideoPlayer.volume;
		}
		
        public function set adVolume(value:Number):void {
			if (value >= 0 && value <= 1) {
				_adVideoPlayer.volume = value;
				dispatchEvent(new VPAIDEvent(VPAIDEvent.AdVolumeChange));
			}
		}
		
        public function get adCompanions():String {
			// No companion creatives
			return "";
		}
		
        public function get adIcons():Boolean {
			return false;
		}
		// ========================
		//  IVPAID Properties end
		// ========================

		// ========================
		//   IVPAID Methods start
		// ========================
        public function handshakeVersion(playerVPAIDVersion:String):String {
			return "2.0";
		}
		
        public function initAd(width:Number, height:Number, viewMode:String, desiredBitrate:Number, creativeData:String="", environmentVars:String=""):void {
			prepare();
			
			// You should respect width and height values here.
			// For simplicity reasons, I didn't resize the creative
			// because I know my demo fits the size well.
			
			// If you are planning on distributing this creative for
			// multiple media sites, you should use environmentVars.
		}
		
        public function resizeAd(width:Number, height:Number, viewMode:String):void {
			// You should implement this method.
			// For simplicity reasons, I know I don't need
			// to resize the creative in this demo and I
			// leave the implementation empty here.
		}
		
        public function startAd():void {
			if (_isStartAdCalled) {
				return;
			}
			
			_isStartAdCalled = true;
			_adVideoPlayer.resume();
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdImpression));
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdStarted));
		}
		
        public function stopAd():void {
			if (_isStopAdCalled || _isSkipAdCalled) {
				return;
			}
			
			_isStopAdCalled = true;
			clean();
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdStopped));
		}
		
        public function pauseAd():void {
			_adVideoPlayer.pause();
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdPaused));
		}
		
        public function resumeAd():void {
			_adVideoPlayer.resume();
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdPlaying));
		}
		
        public function expandAd():void {
			// Cannot be expanded, do nothing.
		}
		
        public function collapseAd():void {
			// Cannot be collapsed, do nothing.
		}
		
        public function skipAd():void {
			// The creative can be skipped all the time.
			// No need to check the adSkippableState property
			// Just go ahead and skip the ad.
			if (_isStopAdCalled || _isSkipAdCalled) {
				return;
			}
			
			_isSkipAdCalled = true;
			clean();
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdSkipped));
			
			// Uncomment the following line if support VPAID prior to 2.0
			// dispatchEvent(new VPAIDEvent(VPAIDEvent.AdStopped));
		}
		// ========================
		//   IVPAID Methods end
		// ========================
	}
	
}
