﻿package  {
	import com.ad.vpaid.VPAIDEvent;
	import com.ad.vpaid.VPAIDWrapper;
	import flash.display.DisplayObject;
	import flash.display.Loader;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.net.URLRequest;
	import flash.utils.Timer;
	import flash.events.TimerEvent;
	
	public class Demo extends MovieClip {

		// Instances on the stages:
		// wrapper_mc
		// time_txt
		// hint_txt
		// textarea
		
		private var _loader:Loader;
		private var _ad:VPAIDWrapper;
		private var _adContent:DisplayObject;
		private var _remainingTimeTimer:Timer;
		
		public function Demo() {
			loadAd();
		}

		private function log(text:String, type:String = ""):void {
			var info:String = "";
			if (type != "") {
				info += "[" + type + "] ";
			}
			info += text + "\n";
			textarea.text += info;
		}
		
		private function loadAd():void {
			_loader = new Loader();
			_loader.contentLoaderInfo.addEventListener(Event.COMPLETE, initAd);
			_loader.load(new URLRequest("./creative.swf"));
		}
		
		private function initAd(event:Event):void {
			try {
				_ad = new VPAIDWrapper(_loader.content);
				if (_ad.handshakeVersion("2.0") == "2.0") {
					// You should have a timer for load timeout
					// in case AdLoaded is never fired.
					// I ignored this step for simplicity.
					_adContent = _loader.content;
					_ad.initAd(640, 400, "normal", -1);
					_ad.addEventListener(VPAIDEvent.AdLoaded, onAdLoaded);
				}
				else {
					textarea.text = "[Error] 当前创意不是 VPAID 2.0 创意";
				}
			}
			catch(error:Error) {
				trace(error.message);
				textarea.text = "[Error] 无法获取创意内容";
			}
		}
		
		private function onAdLoaded(event:VPAIDEvent):void {
			_ad.removeEventListener(VPAIDEvent.AdLoaded, onAdLoaded);
			
			_ad.addEventListener(VPAIDEvent.AdError, onAdError);
			
			_ad.addEventListener(VPAIDEvent.AdStarted, onAdStarted);
			_ad.addEventListener(VPAIDEvent.AdStopped, onAdStopped);
			_ad.addEventListener(VPAIDEvent.AdSkipped, onAdSkipped);
			_ad.addEventListener(VPAIDEvent.AdImpression, onAdImpression);
			
			_ad.addEventListener(VPAIDEvent.AdInteraction, onAdInteraction);
			_ad.addEventListener(VPAIDEvent.AdClickThru, onAdClickThru);
			
			_ad.addEventListener(VPAIDEvent.AdVideoStart, onAdVideoStart);
			_ad.addEventListener(VPAIDEvent.AdVideoFirstQuartile, onAdVideoFirstQuartile);
			_ad.addEventListener(VPAIDEvent.AdVideoMidpoint, onAdVideoMidpoint);
			_ad.addEventListener(VPAIDEvent.AdVideoThirdQuartile, onAdVideoThirdQuartile);
			_ad.addEventListener(VPAIDEvent.AdVideoComplete, onAdVideoComplete);
			
			_ad.addEventListener(VPAIDEvent.AdDurationChange, onAdDurationChange);
			
			wrapper_mc.addChild(_adContent);
			hint_txt.visible = false;
			_remainingTimeTimer = new Timer(500);
			_remainingTimeTimer.addEventListener(TimerEvent.TIMER, updateCountdown);
			_remainingTimeTimer.start();
			_ad.startAd();
		}
		
		private function updateCountdown(event:TimerEvent):void {
			var countdown:int = Math.floor(_ad.adRemainingTime);
			time_txt.text = String(countdown);
		}
		
		private function onAdError(event:VPAIDEvent):void {
			log(event.data.message, event.type);
		}
		
		private function onAdStarted(event:VPAIDEvent):void {
			log("创意开始播放", event.type);
		}

		private function onAdStopped(event:VPAIDEvent):void {
			log("创意播放完成并卸载成功", event.type);

			clean();
			wrapper_mc.removeChild(_adContent);
			hint_txt.text = "创意播放完成";
			hint_txt.visible = true;
		}
		
		private function onAdSkipped(event:VPAIDEvent):void {
			log("创意跳过成功", event.type);

			clean();
			wrapper_mc.removeChild(_adContent);
			hint_txt.text = "创意被跳过";
			hint_txt.visible = true;
		}
		
		private function onAdImpression(event:VPAIDEvent):void {
			log("创意印象，配合 VAST 3 Impression 事件", event.type);
		}
		
		private function onAdInteraction(event:VPAIDEvent):void {
			log("用户正在交互", event.type);
		}
		
		private function onAdClickThru(event:VPAIDEvent):void {
			log("用户点击，需要跳转页面至 " + event.data.url, event.type);
			
			// You should navigate to the URL here
			// if event.data.playerHandles == true
		}
		
		private function onAdVideoStart(event:VPAIDEvent):void {
			log("创意视频开始播放", event.type);
		}
		
		private function onAdVideoFirstQuartile(event:VPAIDEvent):void {
			log("创意视频播放至 1/4 处", event.type);
		}
		
		private function onAdVideoMidpoint(event:VPAIDEvent):void {
			log("创意视频播放至 1/2 处", event.type);
		}
		
		private function onAdVideoThirdQuartile(event:VPAIDEvent):void {
			log("创意视频播放至 3/4 处", event.type);
		}
		
		private function onAdVideoComplete(event:VPAIDEvent):void {
			log("创意视频完成播放", event.type);
			_ad.stopAd();
		}
		
		private function onAdDurationChange(event:VPAIDEvent):void {
			log("创意时长改变，新的时长为 " + String(_ad.adDuration), event.type);
			
			if (_ad.adDuration == -2) {
				_remainingTimeTimer.stop();
			}
			else {
				_remainingTimeTimer.start();
			}
		}

		private function clean():void {
			_remainingTimeTimer.stop();
			_remainingTimeTimer.removeEventListener(TimerEvent.TIMER, updateCountdown);

			_ad.removeEventListener(VPAIDEvent.AdError, onAdError);
			
			_ad.removeEventListener(VPAIDEvent.AdStarted, onAdStarted);
			_ad.removeEventListener(VPAIDEvent.AdStopped, onAdStopped);
			_ad.removeEventListener(VPAIDEvent.AdImpression, onAdImpression);
			
			_ad.removeEventListener(VPAIDEvent.AdInteraction, onAdInteraction);
			_ad.removeEventListener(VPAIDEvent.AdClickThru, onAdClickThru);
			
			_ad.removeEventListener(VPAIDEvent.AdVideoStart, onAdVideoStart);
			_ad.removeEventListener(VPAIDEvent.AdVideoFirstQuartile, onAdVideoFirstQuartile);
			_ad.removeEventListener(VPAIDEvent.AdVideoMidpoint, onAdVideoMidpoint);
			_ad.removeEventListener(VPAIDEvent.AdVideoThirdQuartile, onAdVideoThirdQuartile);
			_ad.removeEventListener(VPAIDEvent.AdVideoComplete, onAdVideoComplete);
			
			_ad.removeEventListener(VPAIDEvent.AdDurationChange, onAdDurationChange);
		}
	}
		
}
