﻿package com.ad.vpaid {
	import com.ad.vpaid.VPAIDEvent;
	
	import flash.display.Sprite;
	import flash.events.AsyncErrorEvent;
	import flash.events.Event;
	import flash.events.TimerEvent;
	import flash.net.NetConnection;
	import flash.net.NetStream;
	import flash.media.SoundTransform;
	import flash.media.Video;
	import flash.utils.Timer;	
	
	public class AdVideoPlayer extends Sprite {

		public static const METADATA_FETCHED:String = "AdMetaDataFetched";
		private const TIMER_STEP:Number = 500;
		
		public var autoPlay:Boolean = true;
		
		private var _isMetaDataApplied:Boolean = false;
		private var _isVideoStarted:Boolean = false;

		private var _video:Video;
		private var _nc:NetConnection;
		private var _ns:NetStream;
		
		private var _width:Number = 320; // the default width of Video instance is 320
		private var _height:Number = 240; // the default height of Video instance is 240
		
		/*
		 * If user sets the duration, ignore videoDuration.
		 * Use this value to calculate video playing progress.
		 */
		private var _duration:Number = 0;
		
		/*
		 * If duration is not set, use this value.
		 * It will be updated once the video metadata is fetched.
		 * If for some reason, the metadata is empty or doesn't
		 * contain a duration, default to 15 seconds.
		 */
		private var _videoDuration:Number = 15;
		private var _volume:Number = 1;
		
		private var _time:Number;
		private var _progressTimer:Timer;
		private var _isFirstQuartilePassed:Boolean = false;
		private var _isMidpointPassed:Boolean = false;
		private var _isThirdQuartilePassed:Boolean = false;
		private var _isVideoPlayCompleted:Boolean = false;
		
		public function AdVideoPlayer() {
		}
		
		public function get video():Video {
			return _video;
		}

		public function get videoWidth():Number {
			return _width;
		}
		
		public function get videoHeight():Number {
			return _height;
		}

		public function set videoWidth(w:Number):void {
			if (_video) {
				_video.width = w;
			}
			_width = w;
		}
		
		public function set videoHeight(h:Number):void {
			if (_video) {
				_video.height = h;
			}
			_height = h;
		}
		
		public function get duration():Number {
			if (_duration > 0) {
				return _duration;
			}
			else {
				return _videoDuration;
			}
		}
		
		public function set duration(value:Number):void {
			if (value > 0) {
				_duration = value;
			}
		}
		
		public function get videoDuration():Number {
			return _videoDuration;
		}
		
		public function get volume():Number {
			return _volume;
		}
		
		public function set volume(value:Number):void {
			if (_ns) {
				var stf:SoundTransform = new SoundTransform();
				stf.volume = value;
				_ns.soundTransform = stf;
			}
		}
		
		public function get time():Number {
			return _time;
		}
		
		public function load(url:String):void {
			resetProgressTimer();
			_isVideoStarted = false;
			_isMetaDataApplied = false;

			_nc = new NetConnection();
			_nc.connect(null);
			_ns = new NetStream(_nc);
			_ns.client = this;
			_ns.play(url);
			_ns.addEventListener(AsyncErrorEvent.ASYNC_ERROR, asyncErrorHandler);
			
			_video = new Video();
			_video.attachNetStream(_ns);
			
			if (autoPlay) {
				onVideoStarted();
			}
			else {
				_ns.pause();
			}
		}
		
		public function clean():void {
			resetProgressTimer();
			_video.attachNetStream(null);
			
			_ns.pause();
			_ns.close();
			_nc.close();
			_ns.removeEventListener(AsyncErrorEvent.ASYNC_ERROR, asyncErrorHandler);
		}
		
		public function seek(offset:Number):void {
			_ns.seek(offset);
		}
		
		public function pause():void {
			_ns.pause();
			_progressTimer.stop();
		}
		
		public function resume():void {
			onVideoStarted();
			_ns.resume();
			_progressTimer.start();
		}
		
        /**
         * http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/net/NetStream.html#event:onMetaData
         */
		public function onMetaData(info:Object):void {
			/*for (var prop:String in info) {
				trace(prop + ' : ' + info[prop]);
			}*/
			if (!_isMetaDataApplied) {
				_isMetaDataApplied = true;
				_width = info.width;
				_height = info.height;
				_videoDuration = Number(info.duration);
				_video.width = _width;
				_video.height = _height;
				
				dispatchEvent(new Event(METADATA_FETCHED));
			}
		}
		
        /**
         * http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/net/NetStream.html#event:onPlayStatus
         * 主要是监听视频是否完整地播放完了		 
         */
		public function onPlayStatus(status:Object):void {
			if (status.code === "NetStream.Play.Complete") {
				
				// 若设置了 duration，则不考虑视频本身是否真的播完。
				if (_duration === 0) {
					dispatchEvent(new VPAIDEvent(VPAIDEvent.AdVideoComplete));
					if (_progressTimer) _progressTimer.stop();
				}

				_time = _ns.time;
                _isVideoPlayCompleted = true;
			}
		}
		
		private function resetProgressTimer():void {
			if (_progressTimer) {
				_progressTimer.reset();
			}
			else {
				_progressTimer = new Timer(TIMER_STEP);
				_progressTimer.addEventListener(TimerEvent.TIMER, trackProgress);
			}
		}
		
		private function onVideoStarted():void {
			if (_isVideoStarted) return;
			_isVideoStarted = true;
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdVideoStart));
			_progressTimer.start();
		}
		
		private function trackProgress(event:TimerEvent):void {
            if (_isVideoPlayCompleted) {
                _time += TIMER_STEP / 1000;
            }
            else {
                _time = _ns.time;
            }
			
            var percent:Number = Math.round(_time / this.duration * 100);

            if (percent >= 25 && !_isFirstQuartilePassed) {
                _isFirstQuartilePassed = true;
                dispatchEvent(new VPAIDEvent(VPAIDEvent.AdVideoFirstQuartile));
            }

            if (percent >= 50 && !_isMidpointPassed) {
                _isMidpointPassed = true;
                dispatchEvent(new VPAIDEvent(VPAIDEvent.AdVideoMidpoint));
            }

            if (percent >= 75 && !_isThirdQuartilePassed) {
                _isThirdQuartilePassed = true;
                dispatchEvent(new VPAIDEvent(VPAIDEvent.AdVideoThirdQuartile));
            }

            if (percent >= 100) {
				_progressTimer.stop();
                dispatchEvent(new VPAIDEvent(VPAIDEvent.AdVideoComplete));
            }
		}

		private function asyncErrorHandler(event:AsyncErrorEvent):void {
			trace("error happened");
			dispatchEvent(new VPAIDEvent(VPAIDEvent.AdError, { message: "无法加载视频" }));
		}
	}
	
}
